package CBAIS::Actions::Action;

###############################################################
# start of global constants

use Data::Dumper;

use MIME::Base64;

require Util::Tracer;

use CBAIS::CBAIS;

=head1 NAME


CBAIS::Actions::Action - Base class for all CBAIS Actions

=head1 SYNOPSIS

require CBAIS:Actions::Action;


$action = CBAIS::Actions::Action->new;

if ($action->check() && $action->execute())
{

    print $action->toString(), " succeeded\n";

}
else
{

    print "Error : ", $action->errormessage();
}

=head1 DESCRIPTION

The CBAIS::Action::Actions class is the base class for all CBAIS Actions. It
defines an interface for checking, executing, verifying, and undoing an
action to a machine. All action methods (check, execute, verify, undo) return
1 on success and 0 on failure. The getErrorMessage method may be used to
obtain information text on the cause of the error. 


=head1 METHODS

The Action interface defines the following abstract following methods which
should be implemented derived classes.

=over 4

=item check() I<abstract>

Check that the action is able to be executed. Returns 1 for success,
otherwise 0. The fact that an action passes its check test does not
necessarily mean it will complete successfully. Some conditions can only be
tested by executing the action. Implementors of this method should only
check conditions which do not depend on a live install (e.g the existence of
a source file in a copy, but not its target directory).

=cut

sub check 
{
    my ($this) = shift;
    $this->{ERROR_MESSAGE} = $WARNING . $this->not_implemented('check');
    return 1;
}

=item execute() I<abstract>

Carry out this action. This action should perform whatever tasks are required
to "execute" this action. If the action is succssfully executed, the value 1
should be returned. Otherwise the value 0 should be returned and the
ERROR_MESSAGE variable be set to some suiteable status text.

=cut

sub execute
{
    my ($this) = shift;
    $this->{ERROR_MESSAGE} = $WARNING . $this->not_implemented('execute') .
        ' ' . $this->toString();
    return 0;
}

=item verify() I<abstract>

verify that this action has been performed. Return 1 if it has, otherwise
0. The getErrorMessage method may be used to get the error message if a 0
value is returned.

=cut

sub verify
{
    my ($this) = shift;
    $this->{ERROR_MESSAGE} = $WARNING . $this->not_implemented('verify') .
        ' ' . $this->toString();
    return 0;
}


=item undo()

This method is optional. If implemented it should provide the ability to undo
whatever actions the execute method carries out. This base class method prints
an error message and returns a 0 value, indicating that the undo action was
not carried out.

=cut

sub undo
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $this->{ERROR_MESSAGE} = $WARNING . $this->not_implemented('undo') .
        ' ' . $this->toString();
    return 0;
}

=back

The following methods are provided to help implementors of the
CBAISS::Actions::Action interface.

=over 4

=item new()

Base class constructor. Sets up error message holder.

=cut


sub new
{
    # FIXME -
    # enable encryption/secure serialisation based on some
    # get host info interface
    # host specifier - how does this work?
    #
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = {};
    if (@_)
    {
        $this = {@_};
    }
    
    $this->{'ERROR_MESSAGE'} = "No Error";
    bless($this, $class);
    return $this;
}

=item check_keys() hashref, key list

check that all of the specified keys are defined in this object (this hash).

=cut

sub check_keys
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my (@keys) = @_;
    my ($key);

    foreach $key (@keys)
    {
        return 0 unless exists($this->{$key});
    }
    return 1;
}

=item getErrorMessage()

return the current value of this actions error message.

=cut

sub getErrorMessage
{
    my ($this) = shift;
    return $this->{ERROR_MESSAGE};
}

=item setInstallRoot() variable ..

Check if CBAIS_INSTALL_ROOT is set. If so prepend it to the specified variables
in this object. Placed here for utility reasons.

=cut

sub setInstallRoot
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    my ($key);
    
    if (defined($ENV{CBAIS_INSTALL_ROOT}))
    {
        foreach $key (@_)
        {
            if (defined($this->{$key}))
            {
                $this->{$key} =  $ENV{CBAIS_INSTALL_ROOT} .'/' . $this->{$key};
                $this->{$key} =~ s://:/:g;
                $tracer->print("fixed install root for $key");
            }
        }
    }
}

sub not_implemented
{
    my ($this, $method) = @_;
    return "$method method not implemented for " . ref($this);
}

sub serialiseData
{
    my ($this) = shift;
    return $this->serialise();
}

=item serialise()

returns a string suitable for subsequent evaling which stores the state of
this object.

=cut


sub serialise
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $Data::Dumper::Purity = 1;
    $Data::Dumper::Indent = 0;
    
    my ($string) = Data::Dumper->Dump([$this], ['this']);
    $tracer->print('serialised object=' . $string);
    return $string;
}

=item deserialise() string I<static (class) method>

eval the specified string which is asumed to have been created with the
serialize method and attempt to create a CBAIS action object. Returns the
created object or 0; 

=cut

sub deserialise
{
    my ($tracer) = new Util::Tracer(@_);
    my ($string) = shift;
    my ($this);
    $tracer->print("About to deserialise string $string");
    $this = eval $string;
    if ($@)
    {
        print STDERR "Error loading action : $@\n";
        return 0;
    }
    $tracer->print("Deserialised=" . $this->toString());
    return $this;
}

=item loadFile() filename I<static (class) method>

Load the specified file and returns it. Handles text and binary
files. If the file is binary, a scalar is returned. If the file is text, it
is returned as an array. 

=cut

sub loadFile()
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this, $key) = @_;
    my ($file);
    my (@file);
    my ($offset) = 0;

    
    if (! -f $this->{$key})
    {
        $this->{ERROR_MESSAGE} = "File $this->{$key} does not exist";
        return 0;
    }
    if (! open(SOURCE, "<$this->{$key}"))
    {
        $this->{ERROR_MESSAGE} = "Unable to open $this->{$key} : $!";
        return 0;
    }
    if (-T $this->{$key})
    {
        @{$this->{DATA}} =<SOURCE>;
        close(SOURCE);
        $tracer->print("read $this->{$key} ($#{$this->{DATA}} lines)");
        $this->{DATATYPE} = 'text';
    }
    else
    {
        my ($line);
        
        $tracer->print("reading binary file");
        while ($status = read(SOURCE, $file, 16384, $offset))
        {
            $offset += $status;
        }
        $tracer->print("read $this->{$key} ($offset bytes)");
        close(SOURCE);
        $this->{DATA} = [$file];
        $this->{DATATYPE} = 'binary';
    }
    return 1;
}


# writes the array apssed n to this->{FILE} and handles
# error status stuff

sub writeFile
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    my ($line);
    
    $tracer->print("Writing $this->{FILE}");
    
    if (! open(FILE, ">$this->{FILE}"))
    {
        $this->{ERROR_MESSAGE} = "Unable to write to $this->{FILE} : $!";
        return 0;
    }
    foreach $line (@{$this->{DATA}})
    {
        print FILE $line;
    }
    if (! close(FILE))
    {
        $this->{ERROR_MESSAGE} = "Unable to close $this->{FILE} : $!";
        return 0; 
    }
    return 1;
}


=item toString()

return a string representation of this action suitable for printing.

=cut

sub toString()
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($string, $key);
    my (@keys) = grep(!/ERROR_MESSAGE|CLASS|ACTION/, keys %{$this});

    $string = $this->{ACTION};
    foreach $key (@keys)
    {
        $tracer->print("processing key $key");
        $string .= " $this->{$key}";
    }
    return  $string; 
}


=back

end of documentation

=cut

1;
