package CBAIS::Actions::Append;

=head1 NAME

CBAIS::Actions::Append - append text to a file

=head1 SYNOPSIS

$append = new CBAIS::Actions::Append(FILE => '/etc/system',
                                     CLASS => 'TestClass');

if ($append->execute())
{
    print "appended data to file /etc/system\n";
}

if ($append->undo())
{
    print "undo appended data to file /etc/system\n";
}

=head1 DESCRIPTION

The B<CBAIS::Actions::Append> class implements a CBAIS action which appends
text to files. Appends are redoable (as long as the file being appended to
has a comment character (string) defined). If a file does not exist it
is created.

The souce file is searched for using a FileLocator. The default semantics are
for a host specific file (suffix specifiers). See L<CBAIS::Path::FileLocator>
for more information.

=head1 METHODS

=head2 new

The constructor. The following hash keys are required to be passed.

=over 4

=item B<FILE>

the file to append to e.g. /etc/system.

=item B<CLASS>

the CBAIS class this action is defined as part of.

The following keys are optional.

=item B<COMMENT>

the char(s) to use for a comment in the file we are appending to. If not
defined defaults to a "#"
    
The following keys are  deduced.

=item B<SOURCE>

host specific source file conaining the text to be appended.

=item B<WRAPPER_PATTERN>

regexp which is used to match wrapper patterns. It is defined as
"</?CBAIS -%- $this->{CLASS}.Append>", with the optional / marking the
beginning/ending of an append section.

=back

=cut



use Carp;

require Util::Tracer;

use CBAIS::CBAIS;

require CBAIS::Util::FileHolder;

use CBAIS::Config::CBAIS;


require CBAIS::Path::FileLocator;

use base("CBAIS::Actions::FileHolder");

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = new CBAIS::Actions::FileHolder(@_);


    my ($locator) = $this->{FILE_LOCATOR};
    my ($cbais_conf) = $this->{CBAIS_CONFIG};

    my ($specifier) = $cbais_conf->getOption('AppendFileSpecifier');
    my ($root) = "append/$this->{CLASS}";
    my ($file);

    $specifier = 'Host' unless $specifier;

    if ($specifier eq 'None')
    {
        $file = $cbais_conf->getProjectRoot('default') . "/$root/$this-{FILE}";
    }
    else
    {
        $file = $locator->getFile($specifier, $root, $this->{FILE});
    }
    
    $this->{SOURCE} = $file;
    if (! ($this->{SOURCE} && -f $this->{SOURCE}))
    {
        $this->{ERROR_MESSAGE} = "No source file found for $this->{FILE}";
    }
    $this->setHolderFile($this->{SOURCE});
    
    $this->{WRAPPER_PATTERN} = "</?CBAIS -%- $this->{CLASS}.Append>";
    
    # default comment char to use in text files is a hash
    if (! defined($this->{COMMENT}))
    {
        $this->{COMMENT} = '#';
    }
    $this->setInstallRoot('FILE');

    # we dson't need these any more, so reset our references to them
    delete $this->{FILE_LOCATOR};
    delete $this->{CBAIS_CONFIG};

    bless($this, $class);
    return $this;
}


=head2 check

Check this action can execute. The existance of the source file is checked,
its readability and whether it is a text file.

=cut

sub check
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($file_holder) = $this->{FILE_HOLDER};

    if ($file_holder->isLoaded() && ! $file_holder->isBinary())
    {
        return 1;
    }
    elsif (! ($this->{SOURCE} && -f $this->{'SOURCE'} && -T _ && -r _))
    {
        if (! $this->{SOURCE})
        {
            $this->{ERROR_MESSAGE} = "Unable to find source for $this->{FILE}";
        }
        else
        {
            $this->{ERROR_MESSAGE} = "Failed -f -T -r for $this->{SOURCE}";
        }
        return 0;
    }
    else
    {
        return 1;
    }
}


# return 0 - on parse error, 1 on no tags, 2 on 2 tags (begin and end)
sub parse_file
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($i);
    my (@file) = `cat $this->{FILE}`;

    $tracer->print("read file $this->{FILE}");
    
    $this->{START} = -1;
    $this->{END} = -1;

    
    LINE : for ($i = 1; $i <= $#file+1; $i++)
    {
        $tracer->print("Processing line $file[$i-1]");
        next LINE unless $file[$i-1] =~ m/$this->{WRAPPER_PATTERN}/;
        if ($this->{START} == -1)
        {
            $this->{START} = $i-1;
            $tracer->print("Found start of append section : line $i");
        }
        else
        {
            $this->{END} = $i-1;
            $tracer->print("Found end of append section : line $i");
            last LINE;
        }
    }
    
    $this->{FILE_CONTENTS} = \@file;

    if (($this->{START} == -1) && ($this->{END} == -1))
    {
        # no matches still OK
        return 1;
    }
    elsif (($this->{START} != -1) && ($this->{END} != -1))
    {
        return 2;
    }
    else
    {
        $this->{ERROR_MESSAGE} = 'Mismatched append text wrapper tags in ' .
            $this->{FILE};
        return 0;
    }
}

=head2 execute

execute this append.

=cut

sub execute
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($file_holder) = $this->{FILE_HOLDER};
    my (@new_text);
    
    return unless $this->check();
    
    if ( -f $this->{FILE})
    {
        # may have been read at other end of serialisation pipe
        if ($file_holder->isLoaded())
        {
            @new_text = $file_holder->getAsArray();
        }
        else
        {
            @new_text = `cat $this->{SOURCE}`;
        }
        my ($parsed_ok) =  $this->parse_file();
        
        if (! $parsed_ok)
        {
            return 0;
        }
        if ($parsed_ok == 2)
        {
            my (@removed);
            $tracer->print("LINES=$#{$this->{FILE_CONTENTS}}, start=" .
                           ($this->{START} + 1) . ", OFFSET=" .
                           ($this->{END} -1 - $this->{START}));
            @removed =
                splice(@{$this->{FILE_CONTENTS}},
                       ($this->{START} + 1),
                       ($this->{END} - 1 - $this->{START}),
                       @new_text);
            $tracer->print("Replaced\n" . join('', @removed) .  " with \n" .
                           join('', @new_text));
            return $this->_write();
        }
        elsif ($parsed_ok == 1)
        {
            return $this->_append();
        }
        else
        {
            return 0;
        }
    }
    else
    {
        $tracer->print("$this->{FILE} does not exist - creating");
        return $this->_append();
    }
}

=head2 verify


This method verifies that an append has taken place and the contents of the
append file are up to date.

=cut

sub verify
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($file_holder) = $this->{FILE_HOLDER};
    
    return unless $this->check();

    if (! -f $this->{FILE})
    {
        $this->{ERROR_MESSAGE} = "$this->{FILE} does not exist";
        return 0;
    }

    my (@new_text);

    if ($file_holder->isLoaded())
    {
        @new_text = $file_holder->getAsArray();
    }
    else
    {
        @new_text = `cat $this->{SOURCE}`;
    }
    
    my ($i, $j);
    my ($parsed_ok) = $this->parse_file();

    if (! $parsed_ok)
    {
        return 0;
    }
    
    if ($parsed_ok == 2)
    {
        if ($#new_text != ($this->{END} - $this->{START} -2))
        {
            $this->{ERROR_MESSAGE} =  "append lengths differ";
            $this->{ERROR_MESSAGE} .= "$#new_text != ($this->{END} - $this->{START} -1";
            return 0;
        }

        $j = 0;
        for ($i = $this->{START} +1; $i < $this->{END}; $i++)
        {
            if ($this->{FILE_CONTENTS}->[$i] ne $new_text[$j++])
            {
                $tracer->print("$this->{FILE}\[$i\] - " .
                               "$this->{FILE_CONTENTS}->[$i] " .
                               " NE $new_text[$j]");
                return 0;
            }
        }
        $tracer->print("All lines of append matched");
        return 1;
    }
    else
    {
        if ($parsed_ok == 0)
        {
            $this->{ERROR_MESSAGE} =
                "Mismatched append text WRAPPERS (!= 2) in $this->{FILE}";
        }
        elsif ($parsed_ok == 1)
        {
            $this->{ERROR_MESSAGE} = "Append not executed"; 
        }
        return 0;
    }
}

=head2 undo

undo an append which has been made to a file. The file is parsed and checked
for the append text of  aprevious append. If found the text is removed.

=cut

sub undo
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    
    if (! -f $this->{FILE})
    {
        $this->{ERROR_MESSAGE} = "$ERROR $this->{FILE} does not exist";
        return 0;
    }

    my ($parsed_ok) = $this->parse_file();
    if (! $parsed_ok)
    {
        return 0;
    }
    
    if ($parsed_ok == 2)
    {
        return 0 unless (($this->{START} != -1) && ($this->{END} != -1));
        splice(@{$this->{FILE_CONTENTS}},
               $this->{START},
               $this->{END} - $this->{START} + 1);
        return $this->_write();
    }
    else
    {
        $this->{ERROR_MESSAGE} =
            "$ERROR append not implemented - unable to undo";
        return 0;
    }
}

# carry out "blind append" - i.e. just do it. Not to be used externally
sub _append()
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($file_holder) = $this->{FILE_HOLDER};

    my (@new_text);
    
    if (! open(FILE, ">>$this->{FILE}"))
    {
        $this->{ERROR_MESSAGE} = "Unable to open $this->{FILE} : $!";
        return 0;
    }

    if (! $file_holder->isLoaded())
    {
        $file_holder->load();
    }
    @new_text = $file_holder->getAsArray();

    print FILE "$this->{COMMENT} <CBAIS -%- $this->{CLASS}.Append>\n";
    print FILE join('', @new_text);
    print FILE "$this->{COMMENT} </CBAIS -%- $this->{CLASS}.Append>\n";
    
    if (! close(FILE))
    {
        $this->{ERROR_MESSAGE} = "Unable to close $this->{FILE} : $!";
        return 0; 
    }
    return 1;
}

# private method - write array passed in to file - overwriting contents
sub _write
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;

    if (! open(FILE, ">$this->{FILE}"))
    {
        $this->{ERROR_MESSAGE} = "Unable to write to $this->{FILE} : $!";
        return 0;
    } 
    print FILE join('', @{$this->{FILE_CONTENTS}});
    if (! close(FILE))
    {
        $this->{ERROR_MESSAGE} = "Unable to close $this->{FILE} : $!";
        return 0; 
    }
    return 1;
}

sub toString
{
    my ($this) = shift;
    return "append $this->{'SOURCE'} -> $this->{FILE}";
}
   
