package CBAIS::Actions::Copy;


=head1 NAME

CBAIS::Actions::Copy - copy a file

=head1 SYNOPSIS

$copy = new CBAIS::Actions::Copy(FILE   => '/etc/passwd',
                                 MODE  => '0644',
                                 OWNER => 'root',
                                 GROUP => 'root',
                                );

if ($copy->execute())
{
    print "copy /etc/passwd\n";
}

if ($copy->verify())
{
    print "The file /etc/passwd is up to date\n";
}


=head1 DESCRIPTION

The B<CBAIS::Actions::Copy> class implements a CBAIS action which copies
files. The specified file is searched for in the CBAIS data tree. The details
of the search are defined by the value of the "CopyFileSpecifiers"  option in
the CBAIS global config. See L<cbais_conf>.


The verify and undo operations are aso supported. The undo operation
may be disabled for certain files (useful for files like /etc/passwd).

The constructor recognises the following keys in addition to those defined
by the B<CBAIS::Actions::MOGAction> class.
                                              

=over 4

=item FILE

The file to be copied. Note that the source file will be searched for
according to the value of the "CopyFileSpecifiers"  option in the CBAIS
global config.  
                  
=item DONT_UNDO

Disable the undo (i.e.removal) operation for this file.

=back

=head1 CONFIG FILE EXAMPLES

The following class file extracts show two example copy operations. The
second example demosntrates a case where the undo opetation has been
disabled.

{
    action     => 'copy',
    owner      => 'mos',
    group      => 'mos',
    mode       => '0644',
    file       => '/etc/ntp/ntp.conf',
},
    
{
    action     => 'copy',
    owner      => 'mos',
    group      => 'mos',
    mode       => '0644',
    file       => '/etc/passwd',
    dont_undo  => 'yes',
},


=head1 SEE ALSO

L<CBAIS::Actions::MOGAction>
    
L<CBAIS::Actions::Action>

=cut


use Carp;
use File::Copy;

require Util::Tracer;

use CBAIS::Config::CBAIS;
use CBAIS::Path::FileLocator;

use base("CBAIS::Actions::MOGAction");

 
sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = CBAIS::Actions::MOGAction->new(@_);

    croak "Missing required key FILE" unless $this->check_keys('FILE');
    
    my ($filename) = $this->{FILE};
    my ($root) = "copy/$this->{CLASS}";

    my ($locator) = $this->{FILE_LOCATOR};
    my ($cbais_conf) = $this->{CBAIS_CONFIG};


    my ($specifier) = $cbais_conf->getOption('CopyFileSpecifier');

    $specifier = 'Host' unless $specifier;
    
    $this->{SPOOL_DIR} =
        $cbais_conf->getOption('LocalSpoolDir', '/var/spool/cbais');

    if ($specifier eq 'None')
    {
        $this->{SOURCE} =
            $cbais_conf->getProjectRoot('default') . "/$root/$filename";
    }
    else
    {
        $this->{SOURCE} = $locator->getFile($specifier, $root, $filename);
    }
    $this->setHolderFile($this->{SOURCE});
    $this->setInstallRoot('FILE');

    # we don't need these any more, so reset our references to them
    delete $this->{FILE_LOCATOR};
    delete $this->{CBAIS_CONFIG};

    bless($this, $class);
    return $this;
}

sub check
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    
    
    # FIXME - add checks for valid group and owner
    if ( -f $this->{'SOURCE'} or $this->{FILE_HOLDER}->isLoaded())
    {
        # we can see source file or have source loaded.
        return 1;
    }
    else
    {
        $this->{ERROR_MESSAGE} = "$this->{SOURCE} does not exist";
        return 0;
    }
}


sub execute
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    my ($file_holder) = $this->{FILE_HOLDER};
    my ($parent_dir) =  &File::Basename::dirname($this->{'FILE'});

    # FIXME - add logic to mkdir parent if necessary
    if ( ! -d $parent_dir )
    {
        $this->{ERROR_MESSAGE} = "dir $parent_dir does not exist";
        return 0;
    }
    if ($file_holder->isLoaded())
    {
        if (! $file_holder->write($this->{FILE}))
        {
            $this->{ERROR_MESSAGE} = $file_holder->getErrorMessage();
            return 0;
        }
    }
    elsif (! copy($this->{'SOURCE'}, $this->{'FILE'}))
    {
        $this->{'ERROR_MESSAGE'} = $this->toString() . " failed : $!";
        return 0;
    }
    return $this->chown_and_chmod($this->{'FILE'});
}

sub undo
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;

    if (defined($this->{DONT_REMOVE}))
    {
        $this->{ERROR_MESSAGE} = "DONT_REMOVE defined - unable to undo copy";
        return 0;
    }
    
    if (! -f $this->{FILE})
    {
        $this->{ERROR_MESSAGE} = "$this->{FILE} is not installed!";   
        return 0;
    }
    elsif (! unlink($this->{FILE}))
    {
        $this->{ERROR_MESSAGE} =  "unable to remove $this->{FILE} : $!";   
        return 0;
    }
    return 1;
}

sub verify
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    my ($remove) = 0;
    my ($file_holder) = $this->{FILE_HOLDER};

    if (! (-f $this->{SOURCE} or $file_holder->isLoaded()))
    {
        $this->{ERROR_MESSAGE} =
            "$this->{FILE} is not installed in cbais data tree";
        return 0;
    }
    
    if (! -f $this->{FILE})
    {
        $this->{ERROR_MESSAGE} = "$this->{FILE} is not installed!";   
        return 0;
    }
    
    my (@output);
    $tracer->print("checking whether file is loaded");
    my ($cmd);
    my ($filename);

    if ($file_holder->isLoaded())
    {
        $tracer->print("file is loaded - writing");

        # FIXME - clean up after ouserlves - maybe name better
        $filename = "$this->{SPOOL_DIR}/$$.cbais";
        if (! $file_holder->write($filename))
        {
            $this->{ERROR_MESSAGE} = $file_holder->getErrorMessage();
            return 0;
        }
        $remove = $filename;
    }
    else
    {
        $filename = $this->{'SOURCE'};
    }
    
    if ( -B $this->{'FILE'})
    {
        @output = `cmp $filename $this->{'FILE'} 2>&1`;  
    }
    else
    {
        @output = `diff $filename $this->{'FILE'} 2>&1`;
    }
    
    my ($retval) = 0xffff & $?;

    unlink $remove if $remove;

    if ($retval == 0)
    {
        return 1;
    }
    else
    {
        $retval >>= 8;
        $this->{ERROR_MESSAGE} = "Files $this->{'SOURCE'} $this->{'FILE'}" .
            " differ:\n\t" . join("\t", @output) . "diff returned : $retval";
        return 0;
    }
}

sub toString
{
    my ($this) = shift;
    return "copy $this->{SOURCE} $this->{FILE} $this->{OWNER}.$this->{GROUP} $this->{MODE_STR}";
}

