package CBAIS::Actions::Mkdir;

=head1 NAME

CBAIS::Actions::Mkdir - make a directory

=head1 SYNOPSIS

$mkdir = new CBAIS::Actions::Mkdir(DIR   => '/var/log/www',
                                   MODE  => '0755',
                                   OWNER => 'root',
                                   GROUP => 'other',
                                   );

if ($append->execute())
{
    print "mkdir /var/log/www\n";
}

if ($append->undo())
{
    print "removd dir /var/log/www\n";
}

=head1 DESCRIPTION

The B<CBAIS::Actions::Mkdir> class implements a CBAIS action which creates
directories. Directory trees may be specified, in which case all directories
in the tree will have the same permissions (owner, group, and mode).

This class implements the undo method. If the initial mkdir resulted in a
directory tree being created the undo will not remove the tree, but only the
final directory. This is a known limitation.



=cut


use Carp;

use File::Path;

require Util::Tracer;

require CBAIS::Actions::MOGAction;

@ISA = qw(CBAIS::Actions::MOGAction);
 
sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = CBAIS::Actions::MOGAction->new(@_);

    croak unless $this->check_keys('DIR');
    $this->setInstallRoot('DIR');

    bless($this, $class);
    return $this;
}

sub check
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    # FIXME - add checks for valid group and owner
    return 1;
}


sub execute
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;  
    my (@created);
    my ($success) = 1;
    if (@created = mkpath($this->{DIR}, 0, $this->{MODE}))
    {
        my ($dir);
        foreach $dir (@created)
        {
            $tracer->print("setting owner/perms on $dir");
            $success = 0 unless $this->chown_and_chmod($dir);
        }
        return $success;
    }
    else
    {
        $this->{ERROR_MESSAGE} = "Unable to create dir $this->{DIR}";
        return  0;
    }
}

sub verify
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;

    if ( -d $this->{DIR})
    {
        return 1;
    }
    else
    {
        $this->{ERROR_MESSAGE} = "$this->{DIR} does not exist";
        return 0;
    }
}

sub undo
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;

    if (rmdir($this->{DIR}))
    {
        return 1;
    }
    else
    {
        $this->{ERROR_MESSAGE} = "Unable to remove $this->{DIR} : $!";
        return 0;
    }
}

sub toString
{
    my ($this) = shift;
    return "mkdir $this->{DIR} $this->{OWNER}.$this->{GROUP} $this->{MODE_STR}";
}
