package CBAIS::Actions::Pkg;

=head1 NAME

CBAIS::Actions::Pkg - tar archive and graft based package versioning

=head1 SYNOPSIS



=head1 DESCRIPTION

The B<CBAIS::Actions::Pkg> class impements a CBAIS Action which installs tar
archive contents, and optionally supports the graft package versioning
system.

The constructor supports the following keys

=over 4

=item B<FILE>

The name of the tar archive to unpack. May be compressed.
    
=item B<UNPACK_DIR>

The directory to unpack the tar archive in.

=item B<GRAFT> (optional)

If defined and set to the value 'yes', specifies that the directory tree
unpacked from this tar archive should be grafted. See L<graft>.

=item B<GRAFT_OPTIONS> (optional - implies GRAFT=yes)

Defines any special graft options. This option implies GRAFT=yes.

=back

=head1 CONFIG FILE EXAMPLES

{
    file          => 'apache-1.3.14.tgz',
    unpack_dir    => '/usr/local/pkgs',
    graft         => 'yes',
    graft_options => '-t /usr/local/mail',
},

{ 
    file          => 'mytar.tgz',
    unpack_dir    => '/home/mydir',
},



=head1 ASSUMPTIONS

The current version relies on the contents of a tarball being in a directory
tree with a top level directory which is the same name as the tar file
without the tar/gzip extension. This limitation will be rmeoved in future
releases which will use the Archive::Tar library to do some intelligent
parsing of the tar archive to provide a more robust implementation of the
undo and verify methods.

=cut

use Carp;

use MIME::Base64;

require Util::Tracer;
use CBAIS::Util::Utils;
require CBAIS::Util::FileHolder;

use CBAIS::CBAIS;

use CBAIS::Config::CBAIS;
require CBAIS::Path::FileLocator;

use base("CBAIS::Actions::FileHolder");

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = new CBAIS::Actions::FileHolder(@_);
    croak "Missing required keys FILE, UNPACK_DIR"
        unless $this->check_keys('FILE', 'UNPACK_DIR');

    my ($locator) = $this->{FILE_LOCATOR};
    my ($cbais_conf) = $this->{CBAIS_CONFIG};

    my ($specifier) = $cbais_conf->getOption('PkgFileSpecifier');
    my ($root) = "pkgs/$this->{CLASS}";
    my ($file);

    $specifier = 'Host' unless $specifier;

    if ($specifier eq 'None')
    {
        $this->{SOURCE} =
            $cbais_conf->getProjectRoot('default') . "/$root/$this-{FILE}";
    }
    else
    {
        $this->{SOURCE} = $locator->getFile($specifier, $root, $this->{FILE});
    }

    if (! ($this->{SOURCE} && -f $this->{SOURCE}))
    {
        $this->{ERROR_MESSAGE} = "No source file found for $this->{FILE}";
    }
    $this->setHolderFile($this->{SOURCE});
    $this->{PKG} = $this->{FILE};
    $this->{PKG} =~ s:\.(tgz|tar\.gz|tar)$::;
    $this->setInstallRoot('UNPACK_DIR');

    #  we dont need these any more, so reset our references to them
    delete $this->{FILE_LOCATOR};
    delete $this->{CBAIS_CONFIG};

    bless($this, $class);
    return $this;
}


sub execute
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($cmd);
    my (@retvals);
    my ($file_holder) = $this->{FILE_HOLDER};
    
    if (! chdir($this->{UNPACK_DIR}))
    {
        $this->{ERROR_MESSAGE} = "$ERROR Unable to cd $this->{UNPACK_DIR} : $!";
        return 0;
    }
    
    if ($file_holder->isLoaded())
    {
        # if (! open(UNPACK, '| tar -xvf -'))
        # doing it this way was hanging on gzip waiting for eof .... 
        if (! $file_holder->write("/tmp/$this->{FILE}"))
        {
            $this->{ERROR_MESSAGE} = $file_holder->getErrorMessage();
            return 0;
        }
        $this->{SOURCE} = "/tmp/$this->{FILE}";
    }
    
    if (! $this->{SOURCE})
    {
        $this->{ERROR_MESSAGE} = "$ERROR unable to find archive $this->{FILE}";
        return 0;
    }
    if (! -f $this->{SOURCE})
    {
        $this->{ERROR_MESSAGE} = "$ERROR $this->{SOURCE} does not exist";
        return 0;
    }
    $cmd = 'tar ';
    if ($this->{SOURCE} =~ m:(\.tgz|tar\.gz):)
    {
        $cmd .= '-zxf ';
    }
    elsif ($this->{SOURCE} =~ m:\.tar:)
    {
        $cmd .= '-xf ';
    }
    else
    {
        $this->{ERROR_MESSAGE} = "unsupported archive type ($this->{SOURCE})";
        return 0;
    }
    $cmd .= $this->{SOURCE};
    $tracer->print("About to unpack using $cmd");
    
    @retvals = &sysCall($cmd);
    if (@retvals == 2)
    {
        $this->{'ERROR_MESSAGE'} = "Unpack of pkg $this->{SOURCE} failed";
        $this->{'ERROR_MESSAGE'} .= " $retvals[0]: ERRNO :$retvals[1]";
        return 0;
    }
    return 1 unless $this->grafting();

    # FIXME - this may need some special stuff with bootstrapping etc
    $cmd = 'perl graft ';
    $cmd .=  "$this->{GRAFT_OPTIONS} " if defined $this->{GRAFT_OPTIONS};
    $cmd .= " -i $this->{UNPACK_DIR}/$this->{PKG}";
    if ((@retvals = sysCall($cmd)) == 2)
    {
        $this->{'ERROR_MESSAGE'} = "Graft of of pkg $this->{'PKG'} failed";
        $this->{'ERROR_MESSAGE'} .= ": $retvals[0]: ERRNO :$retvals[1]";
        return 0;
    }
    else
    {
        return 1;
    }
}

sub undo
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($pkg_root) = "$this->{UNPACK_DIR}/$this->{PKG}";

    if ($this->grafting())
    {
        # FIXME - this may need some special stuff with bootstrapping etc
        $cmd =  "perl graft -d $pkg_root";
        if ((@retvals = sysCall($cmd)) == 2)
        {
            print STDERR
                "Graft of of pkg $this->{'PKG'} failed ... continuing",
                "with removal\n";
        }
    }
    
    if (-d $pkg_root)
    {
        my (@removed) = rmtree($pkg_root);
        if (@removed)
        {
            return 1;
        }
        else
        {
            $this->{ERROR_MESSAGE} = "Unable to remove $pkg_root";
            return 0;
        }
    }
    $this->{ERROR_MESSAGE} = "$this->{PKG} is not installed";
    return 0;
}

sub grafting
{
    my ($this) = shift;
    return 1 if defined $this->{GRAFT_OPTIONS};
    return 1 if (defined $this->{GRAFT} && $this->{GRAFT} eq 'yes');
    return 0;
}


sub toString
{
    my ($this) = shift;
    my ($string) = "cd $this->{UNPACK_DIR}; tar -zxf $this->{SOURCE} ";
    if ($this->grafting())
    {
        $string .= "$this->{GRAFT_OPTIONS} " if defined $this->{GRAFT_OPTIONS};
        $string .= "-i $this->{UNPACK_LOCATION}/$this->{PKG}"; 
    }
    return $string;
}
