package CBAIS::Actions::Rpm;

=head1 NAME

CBAIS::Actions::Rpm - install an RPM package 

=head1 SYNOPSIS

$rpm = new CBAIS::Actions::Rpm(PACKAGE => 'squid'
                               FILE => 'squid-v2.04.rpm');

if ($rpm->execute())
{
    print "installed squid-v2.04 rpm\n";
}

if ($rpm->verify())
{
    print "squid-v2.04 is installed\n";
}
if ($rpm->undo())
{
    print "squid-v2.04 is removed\n";
}



=head1 DESCRIPTION

The B<CBAIS::Actions::Rpm> class implements a CBAIS action which manages RPM
files. It enables the installation, upgrade and removal of RPP packages. It
is not intended as a replacement for thr rpm system.

=head1 SEE ALSO

L<CBAIS::Actions::Action>

=cut

use Carp;

use RPM::Database;
use RPM::Header;

require Util::Tracer;
use CBAIS::Util::Utils;

use CBAIS::Config::CBAIS;
require CBAIS::Path::FileLocator;

use base("CBAIS::Actions::FileHolder");

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = new CBAIS::Actions::FileHolder(@_);
    # my ($host_config) = &getHostConfig();
    my ($cbais_conf) = getCBAISConfig();
    my ($root) = "rpms/$this->{CLASS}";

    croak "Missing required key(s) FILE, PACKAGE" unless
        $this->check_keys('FILE', 'PACKAGE');

    $this->{SOURCE} =
        $cbais_conf->getProjectRoot('default') . "/$root/$this->{FILE}";

    if (defined($ENV{CBAIS_INSTALL_ROOT}))
    {
        $this->{RPM_ROOT} = $ENV{CBAIS_INSTALL_ROOT};
    }

    $this->{SPOOL_DIR} =
        $cbais_conf->getOption('LocalSpoolDir', '/var/spool/cbais');
    $this->setHolderFile($this->{SOURCE});

    # we dont need these any more, so reset our references to them
    delete $this->{FILE_LOCATOR};
    delete $this->{CBAIS_CONFIG};

    bless($this, $class);
    return $this;
}

sub _loadRpmInfo
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    my ($rpm_db) = new RPM::Database;
    my (@results) = $rpm_db->find_by_package($this->{PACKAGE});
    my ($file_holder) = $this->{FILE_HOLDER};
    
    $this->{INSTALLED_RPM} = $rpm_db->find_by_package($this->{PACKAGE});

    if ($file_holder->isLoaded())
    {
        $filename = "$this->{SPOOL_DIR}/$$.cbais.rpm";
        if (! $file_holder->write($filename))
        {
            $this->{ERROR_MESSAGE} = $file_holder->getErrorMessage();
            return 0;
        }
        $this->{ORIGINAL_SOURCE} = $this->{SOURCE};
        $this->{SOURCE} = $filename;
    }
    elsif (! -f $this->{SOURCE})
    {
        $this->{ERROR_MESSAGE} = "$this->{SOURCE} does not exist!";
        return 0;
    }

    $this->{RPM_FILE} = new RPM::Header($this->{SOURCE});
    
    if (! $this->{RPM_FILE})
    {
        $this->{ERROR_MESSAGE} =
            "Error creating RPM::Header for $this->{SOURCE}";
        return 0;
    }
    return 1;
}

sub check
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;

    if (! $this->{RPM_FILE})
    {
        $this->_loadRpmInfo() || return 0; 
    }
    return 1;
}

sub execute
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    my ($in_name, $in_version, $in_release);
    my ($name, $version, $release);
    my ($cmd) = 'rpm';

    if (defined($this->{RPM_ROOT}))
    {
        $cmd .= " --root $this->{RPM_ROOT}";
    }
    
    if (! $this->check())
    {
        return 0; 
    }
    
    if ($this->{INSTALLED_RPM})
    {
        ($in_name, $in_version, $in_release) = $this->{INSTALLED_RPM}->NVR();
        ($name, $version, $release) = $this->{RPM_FILE}->NVR();
        if (($in_name eq $name) && ($in_version eq $version) &&
            ($in_release eq $release))
        {
            print STDERR "Pkg is already installed\n";
            return 1;
        }
        else
        {
            $cmd .= " -U $this->{SOURCE}"
        }
    }
    else
    {
        $cmd .= " -i $this->{SOURCE}"
    }

    $tracer->print("About to execute $cmd");
    my(@retvals) = &sysCall($cmd);

    if (@retvals == 2)
    {
        $this->{'ERROR_MESSAGE'} = "Install of rpm $this->{SOURCE} failed";
        $this->{'ERROR_MESSAGE'} .= " $retvals[0]: ERRNO :$retvals[1]";
        return 0;
    }
    return 1;
}

sub verify
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    my ($in_name, $in_version, $in_release);
    my ($name, $version, $release);
    my ($cmd);
    
    if (! $this->{RPM_FILE})
    {
        $this->_loadRpmInfo() || return 0; 
    }
    
    if ($this->{INSTALLED_RPM})
    {
        ($in_name, $in_version, $in_release) = $this->{INSTALLED_RPM}->NVR();
        ($name, $version, $release) = $this->{RPM_FILE}->NVR();
        if (($in_name eq $name) && ($in_version eq $version) &&
            ($in_release eq $release))
        {
            return 1;
        }
        else
        {
            $this->{ERROR_MESSAGE} = $this->{PACKAGE};
         
            if ($in_version ne $version)
            {
                $this->{ERROR_MESSAGE} .=
                    " version is out of date : ($in_version NE $version)";
            }
            else
            {
                $this->{ERROR_MESSAGE} .=
                    " release is out of date ($in_release NE $release)";
            }
            return 0;
        }
    }
    else
    {
        $this->{ERROR_MESSAGE} = "$this->{PACKAGE} is not installed";
        return 0;
    }
}

sub toString()
{
    my ($this) = shift;
    return "rpm $this->{PACKAGE}";
}
