package CBAIS::Actions::Symlink;

=head1 NAME

CBAIS::Actions::Symlink - create symbolic links

=head1 SYNOPSIS

$symlink =
    new CBAIS::Actions::Symlink(TARGET => '/etc/hosts',
                                LINK=> '/etc/inet/hosts');

if ($symlink->execute())
{
    print "symlinked /etc/inet/hosts -> /etc/hosts\n";
}

if ($symlink->undo())
{
    print "removed symlink /etc/inet/hosts\n";
}

=head1 DESCRIPTION

The B<CBAIS::Actions::Symlink> class implements a CBAIS action which creates
symbolic links. The verify and undo operations are also supported.

The constructor recognises the following keys

=over 4

=item TARGET

The target file (or directory) to link to. Must exists. May be a relative
value (if dir is specified).

=item LINK

The name of the symbolic link. May be absolute or relative to current
directory if DIR key is specified.

=item DIR

A directory to cd to before creating the symbolic link.

=back


=head1 CONFIG FILE EXAMPLES

The following demonstrates a relative and absolute symlink configuration in
a class file.

{
    action => 'symlink',
    target => '/etc/ntp/ntp.conf',
    link   => '/etc/ntp/ntp.symlink',
    
},

{
    action => 'symlink',
    target => '../hosts',
    link   => 'passwd.symlink',
    dir   => '/etc/ntp',

},

=head1 SEE ALSO

L<CBAIS::Actions::Action>

=cut

###############################################
use Carp;

require Util::Tracer;

use CBAIS::CBAIS;

use base("CBAIS::Actions::Link");


#####################################################

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = new CBAIS::Actions::Link(@_);
    bless($this, $class);
    return $this;
}

sub execute
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;

    return unless $this->_chdir();

    if (! -e $this->{TARGET})
    {
        $this->{ERROR_MESSAGE} = "$this->{'TARGET'} does not exist";
        return 0;
    }
    if (! symlink($this->{TARGET}, $this->{LINK}))
    {
        $this->{ERROR_MESSAGE} =
            "Unable to symlink $this->{TARGET}, $this->{LINK} : $!";
        return 0;
    }
    return 1;
}

sub verify
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;

    return unless $this->_chdir();

    if (! -l $this->{LINK})
    {
        $this->{ERROR_MESSAGE} = "$this->{LINK} is not a symlink"; 
        return 0;
    }
    my ($symlink) = readlink($this->{LINK});
    if ($symlink ne $this->{TARGET})
    {
        $this->{ERROR_MESSAGE} =
            "$this->{LINK} is not linked to $this->{TARGET} (-> $symlink)"; 
        return 0;
    }
    return 1;
}

sub undo
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    
    return unless $this->_chdir();
    if (! -l $this->{LINK})
    {
        $this->{ERROR_MESSAGE} = "$this->{LINK} is not a symlink"; 
        return 0; 
    }
    if (! unlink($this->{LINK}))
    {
        $this->{ERROR_MESSAGE} = "Unable to remove this->{LINK}"; 
        return 0;
    }
    return 1;
}

