package CBAIS::Config::Host;

=head1 NAME

CBAIS::Config::Host - model CBAIS host configs

=head1 DESCRIPTION

The B<CBAIS::Config::Host> class handles all host specific configurations
for a CBAIS host. As such it determines the host name service information,
architecture, OS release while allowing values to be overridden.

It handles the passing of updates on to objects such as FileLocators.

=head1 METHODS

=over 4


=cut

use Carp;

require Exporter;

use Data::Dumper;

use Util::Tracer;

use CBAIS::Util::Utils;

use CBAIS::Config::CBAIS;
use CBAIS::Config::Class;
use CBAIS::Config::KickStart;

require CBAIS::Path::FileLocator;


use CBAIS::Actions::ActionFactory;


our(@ISA) = qw(Exporter);

our(@EXPORT) = qw(getHostConfig);


my ($singleton);

sub getHostConfig
{
    if (! $singleton)
    {
        $singleton = new CBAIS::Config::Host(@_); 
    }
    return $singleton;
}

# returns a suitable hash of system info
sub getSystemInfo
{
    #
    # FIXME - come back and get clear on these terms

    my ($os, $host_name, $os_release, $generic_os_release, $kernel_rev,
        $arch, $machine_class, $processor, $platform, $uname); 

    $uname = `uname -a`;
    
    if ($uname =~ m/sunos/i)
    {
        ($os,
         $host_name,
         $os_release,
         $kernel_rev,
         $arch,
         $machine_class,
         $processor,
         $platform) = split(' ', $uname);
    }
    elsif ($uname =~ m/linux/i)
    {
        # linux system
        ($os,
         $host_name,
         $os_release, $shit) = split(' ', $uname, 4);

        $arch = `uname -m`;
        $processor = `uname -p`;
        chomp($arch);
        chomp($processor);
    }
    
    # $generic_arch = &genericArch($arch);
    # $generic_os_release = &majorOSRelease($os_release);

    my ($data) = { 
        ARCHITECTURE => $arch,
        # GENERIC_ARCHITECTURE => $generic_arch,
        OS => $os,
        OS_RELEASE => $os_release,
        #  MAJOR_OS_RELEASE => $generic_os_release,
        HOSTNAME => $host_name,
        PROCESSOR => $processor,
        };
    return $data;
}


# model a host config in CBAIS. If no host is passed will use current host.

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($hostname);
    my ($this);
    my ($href) = &getSystemInfo();

    $tracer->print("created file locator");
    
    $this = {
        %{$href},
        FILE_LOCATOR =>
            new CBAIS::Path::FileLocator(
                                         'OS'          =>$href->{OS},
                                         'OS_RELEASE'  =>$href->{OS_RELEASE},
                                         'HOSTNAME'    =>$href->{HOSTNAME},
                                         'PROCESSOR'   =>$href->{PROCESSOR},
                                         'ARCHITECTURE'=>$href->{ARCHITECTURE},

                                         ),
                CBAIS_CONFIG => getCBAISConfig(),
            };
    
    bless($this, $class);
    $tracer->print("blessed host config");
    if (@_)
    {
        $hostname = shift;
        $this->setHostName($hostname);
    }
    else
    {
        $this->setHostName($this->{HOSTNAME}); 
    }
    # FIXME  - make this configurable as to extent of file specifier ..
    my ($locator) = $this->{FILE_LOCATOR};
    my ($ks_file) = $locator->getSpecificFile('conf', 'KickStart.conf');

    if ($ks_file)
    {
        $this->{KICKSTART_CONF} = new CBAIS::Config::KickStart($ks_file);
    }
    else
    {
        $this->{KICKSTART_CONF} = new CBAIS::Config::KickStart();
    }
    # croak "No config entry found for host $this->{HOSTNAME}"
    #  unless $this->loadConfig();
    return $this;
}

sub _loadConfig
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this, $conf_file) = @_;
    my ($conf, $code);

    my ($locator) = $this->{FILE_LOCATOR};

    my ($file) = $locator->getSpecificFile('conf', $conf_file);

    if ($file)
    {
        $tracer->print("Attempting to load $conf_file");
        $code = "require \'$file\';";
        $conf = eval $code;
        if ($@)
        {
            print STDERR "Error loading config $conf_file : $@\n";
            return 0;
        }
    }
    else
    {
        return 0;
    }
    return $conf;
}


=item loadConfig()

Loads this hosts configuration file.

=cut


sub loadConfig
{
    # fix me
    # make loader parse perl files, grep require, then eval all requires and
    # put together.
    
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($item, $conf);
    my ($root) = "$ENV{CBAIS_ROOT}/conf";

    foreach $item ($this->{FQ_HOSTNAME}, $this->{HOSTNAME},
                   $this->{DOMAIN}, @{$this->{ALIASES}})
    {
        $tracer->print("checking for $root/$item.conf");
        if ( -f "$root/$item.conf")
        {
            my ($code) = 'require \'' . "$root/$item.conf" . '\';';
            $conf = eval $code;
            if ($@)
            {
                print STDERR "Error loading  $root/$item.conf : $@\n";
                return 0;
            }
            if ($conf)
            {
                $tracer->print("Loaded config $root/$item.conf");
                if (defined($conf->{include}))
                {
                    my ($building_conf) = {};
                    foreach $conf_name (@{$conf->{include}})
                    {
                        my ($base_conf) = $this->_loadConfig("$conf_name.conf");
                        if ($base_conf)
                        {
                            $tracer->print("adding $conf_name to conf");
                            $building_conf = {
                                %{$building_conf},
                                %{$base_conf},
                            };
                        }
                    }
                    $conf = { %{$building_conf},
                              %{$conf},
                          };
                }
                $this->{CONFIG} = $conf;
                
                
                if (defined($conf->{projects}))
                {
                    $this->{FILE_LOCATOR}->setProjects(@{$conf->{projects}});
                    $tracer->print("set Projects " .
                                   join(',', @{$conf->{projects}}));
                    
                }
                return 1;
            }
        }
    }
    print STDERR "no config file found for $this->{HOSTNAME}\n";
    return 0;
}

=item createActions

Parses this hosts configuration and any overrides that have been set (via 
the set* methods) and creates the corresponding action objects. Returns 1 on
success, else 0.

=cut

sub createActions
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my (%specifiers);
    my (@classes, @actions);
    my ($class, $action);
    my ($class_conf) = new CBAIS::Config::Class($this->getFileLocator());
    my (@delayed_actions);
    my ($action_factory) =
        new CBAIS::Actions::ActionFactory('FILE_LOCATOR' => $this->getFileLocator(),
                                          'CBAIS_CONFIG' => $this->{CBAIS_CONFIG}
                                          );
    my ($loaded) = 0;
    $this->{ACTIONS} = [];
    my (@defined_actions) = $this->getActions();

    $tracer->print("Defined actions=" . join(',', @defined_actions));
    CLASS : foreach $class ($this->getClasses())
    {
        my (@action_confs);
        if (! $class_conf->loadClassFile($class))
        {
            print STDERR "Unable to load class file for $class\n";
            next CLASS;
        }
        $loaded++;
        @action_confs = $class_conf->getActions(@defined_actions);
        @actions = ($action_factory->createTitleAction($class));
        
        my ($expecting) = $#action_confs +1; # +1 for title action

        ACTION_CONF : foreach $conf (@action_confs)
        {
            if (defined($this->{CONFIG_MATCH}))
            {
                $tracer->print("checking bypass patterns");
                my ($matched) = 0;
                KEY : foreach $key (keys %{$conf})
                {
                    $tracer->print("checking key $key");
                    if ($conf->{$key} =~ m/$this->{CONFIG_MATCH}/)
                    {
                        $matched++;
                        last KEY;
                    }
                }
                if (! $matched)
                {
                    $tracer->print("Bypassing " . Dumper($conf));
                    $expecting--;
                    next ACTION_CONF;
                }
            }
            $action = $action_factory->create($conf);
            if ($action)
            {
                push(@actions, $action);
            }
        }
        # @actions = ($title, $action_factory->create(@action_confs));
        print "Loaded $class config (", $#actions +1, ") actions)\n";
        if ($#actions < $expecting)
        {
            print STDERR "Errors creating action objects \n";
        }
        
        foreach $action (@actions)
        {
            if ($action->isa('CBAIS::Actions::Cmd') &&
                $action->delayTillEnd()) 
            {
                $tracer->print('delaying action ' . $action->toString() .
                               ' till end');
                push(@delayed_actions, $action);
            }
            else
            {
                push(@{$this->{ACTIONS}}, $action);
            }
        }
    }
    push(@{$this->{ACTIONS}}, @delayed_actions) if @delayed_actions;
    return $loaded;
}


=item B<setHostName> hostname

Explicitely set the hostname. The default value is overridden. A call is made
to gethostbyname unless dns lookups are disabled.

=cut

sub setHostName
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($hostname , $aliases, $addrtype, $length, @addresses);
    my ($cbais_host);
    my ($locator) = $this->{FILE_LOCATOR};
    my (@suffixes);
    $this->{HOSTNAME} = shift;

    if ($this->{CBAIS_CONFIG}->getOption('DNSLookups') eq 'on')
    {
        ($hostname , $aliases, $addrtype, $length, @addresses)
            = gethostbyname($this->{HOSTNAME});
    }
    
    if (! defined($hostname))
    {
        # if we couldn't find it in DNS then set to what we were told. This
        #  covers imaginary hosts which do not really exist (i.e.class hosts)
        $hostname = $this->{HOSTNAME};
        $tracer->print("gethostbyame failed - hostname = $this->{HOSTNAME}");
    }
    else
    {
        $tracer->print("Got back $hostname , $aliases, $addrtype, $length, @addresses");

    }
    
    if ($hostname =~ m/\.(\w|-)+\.(\w|-)+$/)
    {
        # fully qualified
        $this->{FQ_HOSTNAME} = $hostname;
        $this->{HOSTNAME}    = $hostname;
        $this->{DOMAIN}      = $hostname;
        $this->{HOSTNAME}    =~ s/\..*$//;
        $this->{DOMAIN}      =~ s/^.+?\.//;
        @suffixes = ($this->{FQ_HOSTNAME}, $this->{HOSTNAME},
                     $this->{DOMAIN});
    }
    else
    {
        $this->{FQ_HOSTNAME} = $hostname;
        $this->{HOSTNAME}    = $hostname;
        $this->{DOMAIN}      = '';

        @suffixes = ($this->{FQ_HOSTNAME});
    }

    if (defined($aliases) &&  $aliases ne '')
    {
        @{$this->{ALIASES}} = split(' ', $aliases);
        $tracer->print('Initial alias list = ' . join(',', @{$this->{ALIASES}}));
        
    }
    if (defined($this->{CONFIG}->{aliases}))
    {
        foreach $alias (@{$this->{CONFIG}->{aliases}})
        {
            if (grep(/$alias/, @{$this->{ALIASES}}) == 0)
            {
                push(@{$this->{ALIASES}}, $alias);
            }
            else
            {
                $tracer->print("Alias $alias already exists");
            }
        }
        push(@suffixes, @{$this->{ALIASES}});
    }
    else
    {
        $tracer->print("no aliases defined for $this->{HOSTNAME}");
        $this->{ALIASES} = [];
        $this->{DOMAIN} = "";
    }
    
    $tracer->print("hostname info: FQHN=$this->{FQ_HOSTNAME}, " 
                   .  "HN=$this->{HOSTNAME}, DOMAIN=$this->{DOMAIN},"
                   . 'ALIASES=' . join(',', @{$this->{ALIASES}}));
    
    $locator->setSuffixes(@suffixes);
    $this->_exportEnv();
}


sub setArchitecture
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($locator) = $this->{FILE_LOCATOR};
    $this->{ARCHITECTURE} = shift;
    $locator->setArchitecture($this->{ARCHITECTURE});
    $this->_exportEnv();
}

sub setOS
{
    my ($this) = shift;
    my ($locator) = $this->{FILE_LOCATOR};
    $this->{OS} = shift;
    $locator->setOS($this->{OS});
    $this->_exportEnv(); 
}

sub setOSRelease
{
    my ($this) = shift;
    my ($locator) = $this->{FILE_LOCATOR};
    $this->{OS_RELEASE} = shift;
    $locator->setOSRelease($this->{OS_RELEASE});
    $this->_exportEnv();
}


# return the FileLocator for this host
sub getFileLocator
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    return $this->{FILE_LOCATOR};
}

sub getArchitecture
{
    my ($this) = shift;
    return $this->{ARCHITECTURE}; 
}

sub getOS
{
    my ($this) = shift;
    return $this->{OS}; 
}

sub getOSRelease
{
    my ($this) = shift;
    return $this->{OS_RELEASE}; 
}

sub getHostName
{
    my ($this) = shift;
    return $this->{HOSTNAME};
}

# return the fully qualified hostname of this machine
sub getQualifiedHostName
{
    my ($this) = shift;
    return $this->{FQ_HOSTNAME};
}


sub getAliases
{
    my ($this) = shift;
    return @{$this->{ALIASES}};
}

sub getDomain
{
    my ($this) = shift;
    return @{$this->{DOMAIN}};
}

# return array of any extra options to process for this host
sub getCBAISOptions
{
    my ($this) = shift;
    if (defined($this->{CONFIG}->{cbais_options}) &&
        @{$this->{CONFIG}->{cbais_options}})
    {
        return @{$this->{CONFIG}->{cbais_options}};
    }
    else
    {
        return ();
    }
}

=item setClasses()

Set the classes defined for the this host. The corresponding classfile
(Classname.cf) must exist in this hosts class file search path or a runtime
error will result when the loadConfig is called.

=cut

sub setClasses
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $this->{CONFIG}->{classes} = [ @_ ];
}

=item getClasses()

get the list of defined classes for this host.

=cut

sub getClasses()
{
    my ($this) = shift;
    return @{$this->{CONFIG}->{classes}};
    
}

=item setActions()

Set the defined action types for the this host.

=cut

sub setActions
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $tracer->print("Setting actions to " . join(',', @_));
    $this->{CONFIG}->{actions} = [ @_ ];
}

=item getActions()

Return a list of the defined action types for the this host.

=cut

sub getActions()
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    if (defined($this->{CONFIG}->{actions}))
    {
        return @{$this->{CONFIG}->{actions}};    
    }
    else
    {
        # FIXME - this may need to be different based on OS. solaris/linux ..
        return ($this->{CBAIS_CONFIG}->getActions());
    }
}

sub setConfigMatch
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $this->{CONFIG_MATCH} = shift;
}

=item getBuiltActions()

Return a list of built actions i.e. objects implementing the
CBAIS::Actions::Action interface.


=cut

sub getBuiltActions
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;

    if (! (defined($this->{ACTIONS}) && @{$this->{ACTIONS}}))
    {
        return 0;
    }
    else
    {
        return @{$this->{ACTIONS}};
    }
}


=item getOSProfile()

Return the OS specific profile of packages to install on a host. A scalar
string is retunred or the value 0.


=cut

sub getOSProfile
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($cbais_config) = $this->{CBAIS_CONFIG};
    my ($profile_file);
    my ($profile);
    my ($root) = "conf/OSProfiles";
    my ($specifier);

    $specifier = $cbais_config->getOption('OSProfileSpecifiers', 'Arch');
    
    return 0 unless defined(($profile = $this->{CONFIG}->{os_profile}));

    $profile_file =
        $this->{FILE_LOCATOR}->getFile($specifier, $root, $profile);

    if (! $profile_file)
    {
        $tracer->print("unable to load OSProfile $profile");
        return 0;
    }
    my ($code) = 'require \'' . "$profile_file" . '\';';
    
    # my ($code) = join('', `cat $root/$item.conf`);
    # my ($code) = "$root/$item.conf"; 
    $conf = eval $code;

    if ($@)
    {
        print STDERR "Error evaling $code : $@\n";
    }
    else
    {
        $tracer->print("Loaded OSProfile $profile");
    }
    return $conf;
}

sub getKeyboard
{
    my ($this) = shift;
    return $this->{CONFIG}->{keyboard};
}

sub getLang
{
    my ($this) = shift;
    return $this->{CONFIG}->{lang};
}

sub getMouse
{
    my ($this) = shift;
    return $this->{CONFIG}->{mouse};
          
}

sub getTimezone
{
    my ($this) = shift;
    return $this->{CONFIG}->{timezone};
}

sub getRootPW
{
    # FIXME - this should come out of some meta conf file
    my ($this) = shift;
    return $this->{CONFIG}->{rootpw};
}

=item getKickStartOption()

This method gets kickstart config file specific options for this host.
Kickstart file entries that could apply to other Operating systems are
accessed via their own get* methods. e.g. getKeyboard().

=cut

sub getKickStartOption
{
  my ($this) = shift;
  my ($option) = shift;
  return (defined($this->{CONFIG}->{$option}) ?
          $this->{CONFIG}->{$option} :
          $this->{KICKSTART_CONF}->get($option));   
}

sub getCBAISServer
{
    # FIXME - this stuff is a bit messy - need to maybe work out install
    # servers based on network or something ..., maybe per project as well
    # need host grouping mechanism.
    my ($this) = shift;
    my ($option) = shift;
    return $this->{CONFIG}->{install_server};
}

sub getCBAISRoot
{
    my ($this) = shift;
    return $this->{CONFIG}->{cbais_root};
}

sub getRebootCmd
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $tracer->print("CONFIG=" . Dumper($this));
    return $this->{CONFIG}->{reboot_cmd};
}

# export env vars containing pertinient host info so scripts executed under
# us can use info if they like

sub _exportEnv
{
    my ($this) = shift;
    $ENV{CBAIS_OS} =  $this->{OS};
    $ENV{CBAIS_FQ_HOSTNAME} =  $this->{FQ_HOSTNAME};
    $ENV{CBAIS_HOSTNAME} =  $this->{HOSTNAME};
    $ENV{CBAIS_DOMAIN} = $this->{DOMAIN};
    $ENV{OS_RELEASE} = $this->{OS_RELEASE}; 
}

sub toString
{
    my ($this) = shift;
    return  Data::Dumper->Dump([$this], ['this']);
}

1;
