package CBAIS::Path::FileLocator;

=head1 NAME

CBAIS::Path::FileLocator - file locator

=head1 SYNOPSIS

    use CBAIS::Path::FileLocator;

    my ($fl) = new CBAIS::Path::FileLocator();

    my ($file) = $fl->getSpecificFile('conf', 'Class.cf');

    print "Found file $file\n";

=head1 DESCRIPTION

The B<CBAIS::Path::FileLocator> object is used for finding operating
system/processor dependent, and host specific versions of files in directory
trees. It uses the file naming conventions shown below.

<project>/<directory_root>/[<os><ps>]<os_version><ps><architecture>/<filename>.<specifier>

<specifier> = <fq_hostname>|<hostname>|<domain_name>|<alias>

=over 4

=item B<sp>

Path separator string. defaults to '::'. (see CBAIS::Options::PathSeparator)

=item B<project>

The project being searched. A project is an arbitrary root directory used to
group together CBAIS data and configs.

=item B<directory_root>

Different file types are generally organiased inside directory roots under a
given project. e.g. 'conf/Classes', 'copy/classname'. 

=item B<os>

The name of the operating system. This is optional. If enabled, the fallback
directory share is appended to the search path.

=item B<os_version>

The  operating system version.

=item B<architecture>

The architecture.
 

=item B<filename>

The file being searched for e.g. 'ClassFile.cf', '/etc/passwd'.

=item B<specifier>

one of the supported specifiers, hostname, domainname, aliases.

=back


=head1 USAGE

    
    use CBAIS::Util::Utils;
    use CBAIS::Path::FileLocator;

    my ($specifier) = new CBAIS::Path::FileLocator();

    my ($file) = $specifier->getSpecificFile('conf/Classes', 'Class.cf');

    if ($file eq '/cbais/default/conf/Classes/Linux::2.2.16-22::i686/Class.cf')
    {
        print "Found file for Linux 2.2.16-22 and i686\n";

    }

    $file = $specifier->getHostSpecificFile("copy/Class", '/etc/passwd');

    if ($file eq "/cbais/default/copy/Class/etc/passwd.avalon")
    {
        print "Found file for host avalon\n";

    }

    my ($sunos_specifier) = new
       CBAIS::Path::FileLocator(OS => 'Sunos',
                               ARCHITECTURE => 'sun4u',
                               GENERIC_ARCHITECTURE => 'sun4',
                               OS_RELEASE => '5.5',
                               MAJOR_OS_RELEASE => '5.x',
                               PROJECTS => ['project1', 'default', ]);

    $sunos_specifier->getHostArchSpecificFile("pkg/local", 'apache.tgz');
    if ($file eq "test_data/default/pkg/local/Linux::2.2.16-22::i686/apache.tgz.avalon")
    {
        print "Found very very specific file\n";
    
    }

=cut

#########################################################

use Data::Dumper;

use Util::Tracer;
use CBAIS::Util::Utils;

use CBAIS::Config::CBAIS;


=head1 METHODS

The FileLocator methods are listed below.

=over 4

=item new()  hash_ref}

creates a new Path Specifier. The hash reference must contain
values for the various search path elements. Values which can be passed are

=over 4

=item OS

the name of the operating system.

=item OS_RELEASE

operating system release

=item MAJOR_OS_RELEASE

Major release (e.g. 5.x)

=item ARCHITECTURE

architecture of CPU (e.g. sun4u, i686)

=item GENERIC_ARCHITECTURE

generic architecture type (e.g. sun4, i386)

=item PROJECTS

list of projects to search in

=item HOSTNAME

name of host

=item OS_ENABLED

If set to a true value, enables the OS name in search paths.

=back

=cut

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    
    my ($this) = {
        # by default have all search path elements enabled 
        OS_ENABLED => 1,
        PROJECTS => [ 'default' ],
        @_
        };

    $this->{GENERIC_ARCHITECTURE} = &genericArch($this->{ARCHITECTURE});
    $this->{MAJOR_OS_RELEASE} = &majorOSRelease($this->{OS_RELEASE});
   
    my ($cbais_conf) = getCBAISConfig();

    $this->{SEPARATOR} = $cbais_conf->getOption('PathSeparator', '::');
    
    bless($this, $class);
    $tracer->print('Created object ' . Dumper($this));
    $this->_buildSearchPath();
    return $this;
}

=item enableOSName

enable the OS name in the search path.

=cut

sub enableOSName
{
    my ($this) = shift;
    $this->{OS_ENABLED} = 1;
    $this->_buildSearchPath();
}

=item disableOSName()

disable the OS name in the search path.

=cut

sub disableOSName
{
    my ($this) = shift;
    $this->{OS_ENABLED} = 0;
    $this->_buildSearchPath();
}

sub _buildSearchPath
{
    #
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($project, $item);
    my (@path);
    my ($separator) = $this->{SEPARATOR};

    $this->{SEARCH_PATH} = [];
    push(@{$this->{SEARCH_PATH}},
         $this->{OS_RELEASE} . $separator . $this->{ARCHITECTURE});
    
    push(@{$this->{SEARCH_PATH}},
         $this->{OS_RELEASE} . $separator . $this->{GENERIC_ARCHITECTURE});

    push(@{$this->{SEARCH_PATH}},
         $this->{OS_RELEASE} . $separator . 'share');
    
    push(@{$this->{SEARCH_PATH}},
         $this->{MAJOR_OS_RELEASE} . $separator . $this->{ARCHITECTURE});

    push(@{$this->{SEARCH_PATH}},
         $this->{MAJOR_OS_RELEASE} . $separator . $this->{GENERIC_ARCHITECTURE});
    
    push(@{$this->{SEARCH_PATH}},
         $this->{MAJOR_OS_RELEASE} . $separator .'share');
    
    if ($this->{OS_ENABLED})
    {
        foreach $item (@{$this->{SEARCH_PATH}})
        {
            $item = $this->{OS} . $separator . $item;
        }
        push(@{$this->{SEARCH_PATH}}, 'share');
    }
    $tracer->print("Search Path\n" .
                   join("\n", @{$this->{SEARCH_PATH}}));
}

sub getSearchPath
{
    my ($this) = shift;
    return @{$this->{SEARCH_PATH}};
    
}


=item setProjects() project ..

Set the project search list for this FileLocator. Note that the
configuration  details for a project are sourced from the CBAIS.conf config
file. 

=cut

sub setProjects
{
    my ($this) = shift;
    my ($tracer) = new Util::Tracer(@_);
    $this->{PROJECTS} = [ @_ ];
    $tracer->print("set projects to " . join(',', @{$this->{PROJECTS}}));
}

=item setArchitecture() architecture

set the architecture. The generic architecture is automatically set.

=cut

sub setArchitecture
{
    my ($this) = shift;
    $this->{ARCHITECTURE} = shift;
    $this->{GENERIC_ARCHITECTURE} = $this->{ARCHITECTURE};
    $this->{GENERIC_ARCHITECTURE} = &genericArch($this->{ARCHITECTURE});
    $this->_buildSearchPath();
}

=item setOS() os

set the operating system.

=cut

sub setOS
{
    my ($this) = shift;
    $this->{OS} = shift;
    $this->_buildSearchPath();
}

=item setOSRelease() os release

set the operating system release. The major OS release number is
automatically set.

=cut

sub setOSRelease
{
    my ($this) = shift;
    $this->{OS_RELEASE} = shift;
    $this->{MAJOR_OS_RELEASE} = &majorOSRelease($this->{OS_RELEASE});
    $this->_buildSearchPath();
}

=item toString()

return a string representation of this object.

=cut

sub setSuffixes
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $this->{SUFFIXES} = [ @_ ];
    $tracer->print("Set suffixs to " . join(',', @{$this->{SUFFIXES}})); 
}

sub toString
{
    my ($this) = shift;
    my ($key, $string);
    foreach $key (sort keys %{$this})
    {
    #    print "processing key $key\n"; 
        if (ref($this->{$key}) eq 'ARRAY')
        {
            $string .= "$key=>{" . join(', ', @{$this->{$key}}) . '}, ';
        }
        else
        {
            $string .= "$key=>$this->{$key}, ";
        }
    }
    $string =~ s/ $//;
    return $string; 
}

=item getSpecificFile() root, file

=over 4

=item root

the root to perform the search in.

=item file

file to search for (maybe be a filepath e.g. '/etc/passwd').

=back

Locate a specific file. The file is specific as far as operating system type
and  release, and processor type information (i.e. CBAIS dir tree structure)
is concerned. File name suffixes are not applied. If no specific files are
found (and the file does not exist), 0 is returned, otherwise the most
specific file found is returned.

=cut

sub getSpecificFile
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this, $root, $file) = @_;
    my ($item, @search_path);

    my ($cbais_conf) = getCBAISConfig();
    my ($project_root, $project);
    
    foreach $project (@{$this->{PROJECTS}})
    {
        $project_root = $cbais_conf->getProjectRoot($project);
        # &CBAIS::Util::Project::getProject($project)->getProjectRoot();
        if (! $project_root)
        {
            print STDERR "project $project not configured in CBAIS.conf!\n";
            return 0;
        }

        foreach $item (@{$this->{SEARCH_PATH}})
        {
            my ($filename) = "$project_root/$root/$item/$file";
            $filename =~ s:/+:/:g;
            $tracer->print("Checking for $filename");
            if (-f $filename)
            {
                $tracer->print("returning $filename");
                return $filename;
            }
        }
    }
    return 0;
}


=item getHostSpecificFile() root, file

=over 4

=item root

the root to perform the search in.

=item file

file to search for.

=back

Locate a host specific file. Only file suffix specifiers are checked in this
method. The architecture and OS version specifiers are ignored.

=cut

sub getHostSpecificFile()
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this, $root, $file) = @_;
    my ($cbais_conf) = getCBAISConfig();
    
    $tracer->print("search list = " . join(',', @{$this->{SUFFIXES}}));

    foreach $project (@{$this->{PROJECTS}})
    {
        $project_root = $cbais_conf->getProjectRoot($project);
           #  &CBAIS::Util::Project::getProject($project)->getProjectRoot();

        my ($suffix);
        my ($filename);
        foreach $suffix (@{$this->{SUFFIXES}})
        {
            $filename = "$project_root/$root/$file.$suffix";
            $filename =~ s:/+:/:g;
            $tracer->print("checking for $filename");
            return $filename if -f $filename;
        }
        $filename = "$project_root/$root/$file";
        $filename =~ s:/+:/:g;
        $tracer->print("checking for $filename");

        if (-f  $filename)
        {
            $tracer->print("returning $filename");
            return  $filename;
        }
    }
    return 0;
}



=item getHostArchSpecificFile() root, file

=over 4

=item B<root>

the root to perform the search in.

=item B<file>

file to search for.

=back

Locate a host specific file. OS, OS version, architecture and file suffix
specifiers are checked in this method. 

=cut

sub getHostArchSpecificFile
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this, $root, $file) = @_;
    my ($item);
    my ($cbais_conf) = getCBAISConfig();
    
    my ($project_root, $project);

    my ($filename);
    PROJECT : foreach $project (@{$this->{PROJECTS}})
    {
        $project_root = $cbais_conf->getProjectRoot($project);

        # $project_root =
          #   &CBAIS::Util::Project::getProject($project)->getProjectRoot();
        
        $tracer->print("Project root for $project = $project_root");
        foreach $item (@{$this->{SEARCH_PATH}})
        {
            $filename = "$project_root/$root/$item/$file";
            
            $filename =~ s:/+:/:g;
            my ($suffix);
                
            foreach $suffix (@{$this->{SUFFIXES}})
            {
                $tracer->print("checking for $filename.$suffix");
                return "$filename.$suffix" if -f "$filename.$suffix";
            }
            last PROJECT if -f $filename;
            # return $filename if -f $filename;
        }
    }
    if ( -f  $filename)
    {
        $tracer->print("returning $filename");
        return $filename;
    }
    return 0; 
}

#  pass in the type of specifier you want - Arch, Host, or Both
sub getFile
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this, $type, $root, $file) = @_;

    $tracer->print("Looking for \"$type\" specific version of $file");
    
    if ($type eq 'Arch')
    {
        return $this->getSpecificFile($root, $file);
    }
    elsif ($type eq 'Host')
    {
        return $this->getHostSpecificFile($root, $file);
    }
    elsif ($type eq 'Both')
    {
        return $this->getHostArchSpecificFile($root, $file);
    }
    return 0;
}

=back


=cut
    
1;
