package CBAIS::Util::UIDCache;

use CBAIS::CBAIS;

#
# $Id: UIDCache.pm,v 1.4 2001/06/20 08:19:39 mos Exp $
#
=head1 NAME

CBAIS::Util::UIDCache - cache user and group ids during install

=head1 DESCRIPTION

The B<CBAIS::Utils::UIDCache> provides a cache of user and group names during
a machine install.

=head1 METHODS

=over 4

=item B<getUID> ( username )

Returns the numeric user id associated with I<username>.

=item B<getGID> ( groupname )

Returns the numeric group id associated with I<groupname>.

=back

=head1 NOTES
    
A race condition exists where a directory is to be created which will be
owned by a user that will exist only in the local passwd file. This is
because directories are made before copy is done (since you might want to
copy a file into a directory you just created.)
    
To get around this the cache may be "primed" with user and group
id\'s. The priming is done in the file I</CBAIS_ROOT/conf/UIDCache.conf> by
assigning values in the %uids and/or %gids hashes.

Note that the two hashes %uids and %gids must be global variables in the
I<CBJS::Utils::UIDCache> package...

=cut

require Util::Tracer;

require Exporter;

our @ISA = qw(Exporter);

our @EXPORT = qw($NOBODY_UID $NOBODY_GID getUID getGID);

# Constants
#
*NOBODY_UID = \60001;
*NOBODY_GID = \60001;
#
# Global single copy of map
#my (%uids) = ( 'nobody'	=>	$NOBODY_UID, );
#my (%gids) = ( 'nobody'	=>	$NOBODY_GID, );

%uids = ( 'nobody'	=>	$NOBODY_UID, );
%gids = ( 'nobody'	=>	$NOBODY_GID, );

# $uids{'nobody'} = $NOBODY_UID;
# $gids{'nobody'} = $NOBODY_GID;

eval "require 'UIDCache.conf'";

if ($@)
{
    print STDERR "Unable to load UIDCache.conf :$@ \n";
}
#my ($xx);
#foreach $xx (keys %uids) { print "key=$xx, uid=$uids{$xx}\n"; }
#foreach $xx (keys %gids) { print "key=$xx, gid=$gids{$xx}\n"; }

sub getUID
{
    my ($tracer) = new Util::Tracer(@_);

    my ($owner) = shift;
    if (checkUID($owner))
    {
	return $uids{$owner};
    }
    else
    {
        # FIXME - add warning logging 
	$tracer->print("user $owner not found, setting to nobody");
	return $uids{'nobody'};
    }
}

sub getGID
{
    my ($tracer) = new Util::Tracer(@_);

    my ($group) = shift;
    if (&checkGID($group))
    {
	return $gids{$group};
    }
    else
    {
	$tracer->print("group $group not found, setting to nobody");
	return $gids{'nobody'};
    }
}

# Takes a userid as a string and returns 1 if the string maps to a
# valid userid, otherwise -1.
#
sub checkUID
{
    my ($owner) = shift;
    my ($uid) ;
    # if (! defined($this{'UIDS'}{$owner}))
    if (! defined($uids{$owner}))
    {
	$uid = getpwnam($owner);
	if (!defined($uid))
	{
            print STDERR "$WARNING user $owner not found\n";
	    return 0;
	}
	else
	{
	    $uids{$owner} = $uid;
	}
    }
    return 1;
}


# private
sub checkGID
{
    my ($group) = shift;
    my ($gid);
    if (! defined($gids{$group}))
    {
	$gid = getgrnam($group);
	if (! defined($gid))
	{
	    print STDERR "$WARNING group $group not found\n";
	    return 0;
	}
	else
	{
	    $gids{$group} = $gid;
	}
    }
    return 1;
}

1;


