=head1 NAME

Tracer.pm - Debugging - Conditional tracing package

=head2 REVISION

Version 1.2

=head2 DEPENDENCIES

FileHandle

=head1 SYNOPSIS

     require Util::tracer;

     sub some_func {
          my($tracer) = new Util::tracer(@_);

          ...

          $tracer->print("Some debugging");

     }

=head1 DESCRIPTION

This package provides a perl copnditional debug/trace mechanism. No debug is 
produced unless the environment variable TRACE is set. When set, all 
debugging is printed out unless the TRACE variable is set to a string. In 
this case, the string is used as a perl regex to match against the names of 
functions (as passed as the first argument to the tracer constructor) to 
determine whether tracing will be output from those functions.

The trace ouput can be redirected to a file by defining the environment 
variable TRACE_FILE. Otherwise, output goes to standard error.

For perl cgi scripts, where the environment is not under the programmers 
control, the environement variables can be set programatically before 
requiring the package:

$ENV{'TRACE'} = ".*";
$ENV{'TRACE_FILE'} = "/tmp/cgi_out.log";
$ENV{'TRACE_OPTIONS'} = "aspflt";
require Util::tracer;
sub foo {
     my($tracer) = new Util::tracer("foo" @_);
}

The trace package automatically prints out enter and exit lines when a 
tracer
object is created and destroyed. All output done via print is indented
according to the level of nesting of traced functions.

The variable TRACE_OPTIONS also controls what information is printed out. By
default, all that is printed is the function name on entry and exit and
any text passed to the print function. possible options are:

  a: Print the arguments passed to the function
  s: Print a full stack backtrace on entry to any traced function
  d: Indicate the stack depth on any print statements
  p: Print the last package name in the scope with the function name
  P: Print the entire package name
  n: Print the line number of the print as a prefix (overridden by l).
  f: Print the function name with each call to "print"
  l: Print the file name and line no of where the funcwas called from and
    where each call to print is
  L: Print the Full path name of the File on entry to the function
  t: Print the time the trace occurred

p and P and l and L are mutually exclusive, but P implies p and L implies l.

To set: setenv TRACE_OPTIONS asPflt
   or : $ENV{'TRACE_OPTIONS'} = "asPflt";

=cut

use FileHandle;
use POSIX qw(strftime);
package Util::TracerCfg;

initOptions(defined($ENV{'TRACE'}),
            $ENV{'TRACE'},
            $ENV{'TRACE_FILE'},
            $ENV{'TRACE_OPTIONS'});

sub initOptions
{
    my ($trace, $trace_r, $trace_file, $trace_options) = @_;
    $indent = 0;
    $args = 0;
    $func = 0;
    $line = 0;
    $linenum = 0;
    $pack = 0;
    $stack = 0;
    $time = 0;
    $depth = 0;
    undef %cache;
    $traceon = defined $trace && $trace;
    $trace_regexp = defined $trace_r ? $trace_r : '';

    if (defined $trace_file && $trace_file ne ''){
        open(TRACE_FILEH, "> $trace_file") ||
            die "Couldn't open TRACE_FILE: $trace_file :$!\n";
    } else {
        open(TRACE_FILEH, ">&STDERR");
    }
    if (defined $trace_options && $trace_options ne ''){
        $args = $trace_options =~ /a/;
        $func = $trace_options =~ /f/;
        $linenum = $trace_options =~ /n/;
        $line = $trace_options =~ /l/;
        $line = 2 if $trace_options =~ /L/;
        $pack = $trace_options =~ /p/;
        $pack = 2 if $trace_options =~ /P/;
        $stack = $trace_options =~ /s/;
        $time = $trace_options =~ /t/;
        $depth = $trace_options =~ /d/;
    }
    local($oldf) = select TRACE_FILEH;
    $| = 1;
    select $oldf;
}
sub indent {
    my($level) = @_;
    my $indentstr = "";
    my $i = $indent;
    if (defined $level) { $i = $level; }
    for (;$i > 0; $i--) {
     $indentstr .= " ";
    }
    return $indentstr
}

sub new {
    my ($type,$subr,$filename) = @_;
    my $callerDesc = $subr;
    my $funcn = $callerDesc;
    if (!$pack || $func){
     # !$pack : dont show package name
     # $func: show func name in print
     if ($callerDesc =~ /.*:([^:]+)$/){
         if (!$pack){
          $callerDesc = $1;
         }
         $funcn = $1;
     }
    }
    if ($pack == 1){
     # Only show last package name
     if ($callerDesc =~ /:([^:]+::[^:]+)$/){
         $callerDesc = $1;
     }
    }
    if ($line){
     if ($filename =~ /\/([^\/]+)$/){
         $filename = $1;
     }
    }
    my $self = [ ($traceon && ($subr =~ /${trace_regexp}/)) || 0,
                 $callerDesc, $funcn, $filename ];
    bless $self, $type;
    return $self;
}

sub printIn {
    my $self = shift;
    my $filename = shift;
    my $lineno = shift;
    my @args = @_;

    if ($traceon && $self->[0]) {
        print TRACE_FILEH indent() . "TRACE (->) : " ;
     my $argstring = "";
     if ($args){
         # handle the arg string - indent newlines in it.
         $argstring = join(", ", @args);
         my $ind = indent() . " ->" . indent(1);
         $argstring =~ s/\n/\n$ind/g;
     }
     if ($line == 2){
         # show line numbers?
         print TRACE_FILEH "At: $filename($lineno)\n";
         print TRACE_FILEH indent() . "TRACE (->) : " ;
     }
     if ($time){
         # print the current time
         print TRACE_FILEH localtime() . "; ";
     }
     print TRACE_FILEH "$self->[1]($argstring)\n";
     if ($stack){
         # Print full stack dump...
         my $ind = indent() . " ->" . indent(1);
         $self->printStackTrace(2, $ind);
     }
        $indent++;
    }
}

sub printOut {
    my ($self) = @_;
    if ($traceon && $self->[0]) {
        $indent--;
        print TRACE_FILEH indent() . "TRACE (<-) : " ;
     if ($time){
         # print the current time
         print TRACE_FILEH localtime() . "; ";
     }
        print TRACE_FILEH "$self->[1]()\n";
    }
}

sub printStackTrace {
    my($self, $frames, $indent) = @_;
    my($p, $f, $l, $s);
    while (($p, $f, $l, $s) = caller($frames)){
     print TRACE_FILEH $indent . "$s($f:$l)\n";
     $frames++;
    }
}

sub print {
    my ($self, $string) = @_;
    if ($self->[0]) {
        my $prefix;
        if ($time){
            # print the current time
            $prefix .= POSIX::strftime("%T", localtime()) . ' ';
        }
        if ($func){
            # print the short func name
            $prefix .= $self->[2] . "(";
        }
        if ($line){
            # print the short file and line no.
            my($package, $filename, $lineno) = caller(1);
            $prefix .= "$self->[3]:$lineno";
        } elsif ($linenum){
            # print the line number of the print
            my($package, $filename, $lineno) = caller(1);
            $prefix .= $lineno;
        }
        if ($func){
            $prefix .= ")";
        }
        $prefix .= stackDepth(2);
        if ($prefix eq ''){
            $prefix = indent();
        } else {
            $prefix = indent() . $prefix;
        };
        $string =~ s/\n/\n$prefix/g;
        print TRACE_FILEH $prefix, $string, "\n";
    }
}

sub stackDepth {
    if (!$depth) {
        return '';
    }
    my ($fromDepth) = @_;
    my $depth = 0;
    while (caller($depth)){
        $depth++;
    }
    return
    substr(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>",
           0, $depth - $fromDepth);
}

package Util::Tracer;

=item <new(@args)>

Create a new Tracer object. Conditionally prints out a debug trace line 
indicating the function called and the arguments passed to it.

=cut

sub new {
    my $type = shift;
    my @args = @_;
    my ($p, $f, $l, $subr) = caller(1);
    my ($pp, $filename, $lineno) = caller(0);
    if (!defined($subr)){
     $subr = "main";
    }
    my $self = $cache{$subr};
    if (!defined($self)){
     $self = new Util::TracerCfg($subr, $filename);
     $cache{$subr} = $self;
    }
    my $me = [$self];
    $self->printIn($filename, $lineno, @args);
    bless $me, $type;
    return $me;
}

=item <DESTROY>

Called automatically when the Tracer object is destroyed. Conditionally 
prints a debug line indicating the function has been left.

=cut

sub DESTROY {
    my ($me) = @_;
    # sometimes when a Tracer was newed in a script at the top
    # level (i.e. 'main')a dn perl seems to destroy the TracerCfg objects
    # before the Tracer objects that refer to them!!!
    if (defined($me) && defined($me->[0])) {
        $me->[0]->printOut();
    }
}

=item <print($string)>

Print an indented debug string conditionally on tracing enabled for this 
function.

=cut

sub print {
    my ($me, $string) = @_;
    $me->[0]->print($string);
}

=item <initOptions()>

Set up the Tracer options.
Arguments:
  traceon: 1 to turn tracing on
  trace_regexp: regexp of function names to trace in - '' for all
  trace_file: file to output tracing to, '' for stderr
  trace_options: as TRACE_OPTIONS environment variable.
This overrides previous settings via envoronment variables.    

=cut

sub initOptions
{
  Util::TracerCfg::initOptions(@_);
}
